require_dependency "binect_importer/application_controller"
require_dependency "../../lib/binect_importer/dropbox_importer"

require 'dropbox_sdk'
require 'securerandom'

# This configuration will be added by the Rails-Application
import_config = Rails.application.config.import_config[:dropbox]
APP_KEY       = import_config[:APP_KEY]
APP_SECRET    = import_config[:APP_SECRET]
ROOT_PATH     = import_config[:ROOT_PATH]

module BinectImporter
    class DropboxRegistrationsController < AuthenticatedController
        def start_registration
            authorize_url = get_web_auth().start()
            Rails.logger.info "Start Dropbox registration for user: #{current_user}. Now redirect to URL: #{authorize_url}"

            redirect_to authorize_url
        end

        # GET /import_registration/dropbox_finish
        def finish_registration
            begin
                access_token, user_id, url_state = get_web_auth.finish(params)

                Rails.logger.info "Registration succeeded for user: #{current_user}."

                # Check whethere this user is already authorized
                importer_data_index = current_user.authenticated_importers.index { |importer_hash| importer_hash[:type] == DropboxImporter::IMPORTER_TYPE }
                importer_data = { class_name: "BinectImporter::DropboxImporter", type: DropboxImporter::IMPORTER_TYPE, access_token: access_token, user_id: user_id, url_state: url_state }
                if importer_data_index.nil?
                    # Add new importer_data
                    Rails.logger.info "Saving new access token"
                    current_user.authenticated_importers.push(importer_data)
                else
                    # Replace importer data
                    Rails.logger.info "Updating new access token"
                    current_user.authenticated_importers[importer_data_index] = importer_data
                end
                unless current_user.save
                    Rails.logger.error  "Unable to persist change: #{current_user.errors}"
                    flash[:error] = "Unable to save data. Please try again"
                end
                redirect_to ROOT_PATH
            rescue DropboxOAuth2Flow::BadRequestError => e
                flash[:error] = "Error in OAuth 2 flow: Bad request: #{e}"
                redirect_to ROOT_PATH
            rescue DropboxOAuth2Flow::BadStateError => e
                logger.info("Error in OAuth 2 flow: No CSRF token in session: #{e}")
                redirect_to(:action => 'start_registration')
            rescue DropboxOAuth2Flow::CsrfError => e
                logger.info("Error in OAuth 2 flow: CSRF mismatch: #{e}")
                flash[:error] = "CSRF error"
                redirect_to ROOT_PATH
            rescue DropboxOAuth2Flow::NotApprovedError => e
                flash[:error] = "Not approved?  Why not, bro?"
                redirect_to ROOT_PATH
            rescue DropboxOAuth2Flow::ProviderError => e
                logger.info "Error in OAuth 2 flow: Error redirect from Dropbox: #{e}"
                flash[:error] = "Strange error."
                redirect_to ROOT_PATH
            rescue DropboxError => e
                logger.info "Error getting OAuth 2 access token: #{e}"
                flash[:error] = "Error communicating with Dropbox servers."
                redirect_to ROOT_PATH
            end
        end

        def unlink
            client = get_dropbox_client
            if client.nil?
                flash[:error] = "Unlinking not possible."
            else
                # TODO: Invalidate token.
                importer_data_index = current_user.authenticated_importers.index { |importer_hash| importer_hash[:type] == DropboxImporter::IMPORTER_TYPE }
                current_user.authenticated_importers.delete_at(importer_data_index)
                current_user.save
                flash[:success] = "Unlinking successfully"
            end
            redirect_to ROOT_PATH
        end

        private
        def get_web_auth
            return DropboxOAuth2Flow.new(APP_KEY, APP_SECRET, url_for(:action => 'finish_registration'),
                                         session, :dropbox_auth_csrf_token, "de")
        end

        def get_dropbox_client
            DropboxImporter::get_dropbox_client(current_user)
        end

    end
end
