require 'google/apis/drive_v3'
require 'google/api_client/client_secrets'

FOLDER_NAME = "Digipapr"

# see:  https://developers.google.com/drive/v3/web/folder
#       http://www.rubydoc.info/github/google/google-api-ruby-client/Google/Apis/DriveV3/DriveService
module BinectImporter
    class GoogledriveImporter
        IMPORTER_TYPE = "googledrive"

        def initialize(user)
            @user = user
        end

        # Count files in folder..
        def count
            drive = drive_service
            folder_id = create_folder( FOLDER_NAME, drive )

            page_token = nil
            count = 0
            begin
                response = drive_service.list_files(q: "mimeType='application/pdf' and '#{folder_id}' in parents",
                                                    spaces: 'drive',
                                                    fields:'nextPageToken, files(id, name)',
                                                    page_token: page_token)
                count += response.files.count
            end while !page_token.nil?

            count
        end

        def import
            drive = drive_service
            folder_id = create_folder( FOLDER_NAME, drive )

            page_token = nil
            begin
                response = drive_service.list_files(q: "mimeType='application/pdf' and '#{folder_id}' in parents",
                                                    spaces: 'drive',
                                                    fields:'nextPageToken, files(id, name)',
                                                    page_token: page_token)
                for file in response.files
                    # Process change
                    puts "Found file: #{file.name} #{file.id}"
                    Rails.logger.info "Import PDF: Load file: #{file.name} #{file.id}"

                    # Load Meta-Data
                    file = drive_service.get_file(file.id)

                    # Load File-Content
                    file_buffer = drive_service.get_file(file.id, download_dest: StringIO.new)

                    Rails.logger.info "Import PDF: ready"

                    # Call external block that will handle the file.
                    is_success = yield( file_buffer, file.name, file.mime_type, file_buffer.size )

                    if is_success
                        # Remove file
                        Rails.logger.info "Remove file: file: #{file.name} #{file.id}"
                        drive_service.delete_file(file.id)
                    end
                end
                page_token = response.next_page_token
            end while !page_token.nil?
        end

        def export(file_data, file_path, mime_type)
            Rails.logger.info "Exporting file '#{file_path}'.."
            drive = drive_service
            # First create/get the main folder
            digipapr_folder_id = create_folder( FOLDER_NAME, drive )

            # now find sub folder
            path = File.dirname(file_path)
            folder_id = create_folder( path, drive, digipapr_folder_id )

            metadata  = {
                name: File.basename(file_path),
                parents: [folder_id]
            }

            gd_file = drive.create_file(metadata, fields: 'id', upload_source: StringIO.new(file_data), content_type: mime_type)

            if gd_file.present?
                Rails.logger.info "Created file with id '#{gd_file.id}'"
                return true
            else
                Rails.logger.info "Unable to create file"
            end

            return false
        end

        private
        # Creates the given folder if its exists.
        # @return the folder id
        def create_folder( folder_name, drive_service, parent_folder_id = nil )
            # remove leading "/" if it exists..
            folder_name[0] = '' if folder_name[0] == "/"
            puts "Looking for: #{folder_name} in parent #{parent_folder_id}"
            folder_id = nil

            # First check whether the folder exists.
            page_token = nil
            begin
                # A folder is a file with a special mimeType
                subfolder_query = "and 'root' in parents"
                if parent_folder_id.present?
                    subfolder_query = "and '" + parent_folder_id + "' in parents"
                end

                response = drive_service.list_files(q: "mimeType='application/vnd.google-apps.folder' #{subfolder_query}", spaces: 'drive', fields:'nextPageToken, files(id, name, trashed)', page_token: page_token)
                for file in response.files
                    # Process change
                    #puts "Found file: #{file.name} #{file.id}"
                    next if file.name != folder_name
                    next if file.trashed
                    # Found file
                    folder_id = file.id
                end

                page_token = response.next_page_token
            end while !page_token.nil?

            # Create the folder if it not exists.
            if folder_id.nil?
                # Create folder
                folder_metadata = {
                    name: folder_name,
                    mime_type: 'application/vnd.google-apps.folder'
                }
                folder_metadata[:parents] = [parent_folder_id] if parent_folder_id.present?
                folder = drive_service.create_file(folder_metadata, fields: 'id')
                folder_id = folder.id
            end

            folder_id
        end

        def drive_service
            importer_data = @user.authenticated_importers.select { |importer_hash| importer_hash[:type] == GoogledriveImporter::IMPORTER_TYPE }
            auth_data = importer_data.first[:access_data]

            client_opts = JSON.parse(auth_data)
            auth_client = Signet::OAuth2::Client.new(client_opts)
            auth_client.fetch_access_token!
            drive = Google::Apis::DriveV3::DriveService.new
            drive.authorization = auth_client

            # Store access data in order avoid outdated access tokens
            importer_data_index = @user.authenticated_importers.index { |importer_hash| importer_hash[:type] == GoogledriveImporter::IMPORTER_TYPE }
            importer_data = { class_name: "BinectImporter::GoogledriveImporter", type: GoogledriveImporter::IMPORTER_TYPE, access_data: auth_client.to_json }
            @user.authenticated_importers[importer_data_index] = importer_data
            @user.save

            drive
        end
    end


end
